/********************************************************/
/* Copyright (C) 2001 OSS Nokalva.  All rights reserved.*/
/********************************************************/

/* THIS FILE IS PROPRIETARY MATERIAL OF OSS Nokalva, INC. AND
 * MAY BE USED ONLY BY DIRECT LICENSEES OF OSS Nokalva, INC.
 * THIS FILE MAY NOT BE DISTRIBUTED. */

/* Prepared by OSS Nokalva, Inc.  11/22/2000      */

/* Application Program: Usebcd.java
   Encodes/Decodes/Prints BCD Abstract Syntax
    Demonstrates the UserClass directive of the OSS ASN.1/Pure Java Tools
    API using bcd.asn and MyBCD.java. To run this program:

    asn1pjav bcd.asn
    move MyBCD.java bcd
    cd bcd
    bcd.bat javac
    cd ..
    javac -g Tbcd.java
    java Tbcd
 */

/* The user defined class MyBCD is used as a replacement for
 * the compiler-generated class BCDString. MyBCD allows the encoded
 * version of the value to be in binary coded decimal (BCD) format,
 * while allowing the value at any time to be printable in
 * ascii (readable) format.
 *
 * This program creates, encodes, and decodes a Sequence type
 * called BBCard which contains a MyBCD value. The value is printed out
 * at each step. Further this program demonstrates how the user class can
 * be created with non-ascii values, but still print, and how an illegal
 * value will raise an exception.
 */

/* Compiler-generated classes */
import bcd.*;
import bcd.a.*;

/* Universal classes from the OSS runtime library */
import com.oss.asn1.*;
import com.oss.util.*;

/* Java I/O classes */
import java.io.*;

public class Tbcd {

    public static void main(String args[]) {

	/*******************************/
	/* Do OSS API initializations. */
	/*******************************/
	try {
	    Bcd.initialize();
	} catch (Exception e) {
	    System.out.println("Initialization exception: " + e);
	    System.exit(1);
	}

	Coder coder = Bcd.getBERCoder();

	/* Turn on trace capabilities */
	coder.enableEncoderDebugging();
	coder.enableDecoderDebugging();

	/*******************************************/
	/* Create the value to encode using decimal*/
	/* notation for the BCD number             */
	/*******************************************/
	BBCard myCard = new BBCard(
	    new MyBCD("12345678901"));

	/*******************************************/
	/* Print the intended input to the encoder */
	/*******************************************/
	System.out.println("\nThe input to the encoder is:");
	System.out.println("(Note that the value of BBCard.a is printed " +
	    "in the custom display format)\n");
	System.out.println(myCard);

	/*******************************************/
	/* Print the value of the field in decimal */
	/* notation.                               */
	/*******************************************/
	System.out.println("The value of field 'a' has been set to " +
	    myCard.getA());

	/************************************/
	/* Call the encoder to encode myMsg */
	/************************************/
	ByteArrayOutputStream sink = new ByteArrayOutputStream();

	/* Encode the data.  Exception on failure.              */
	System.out.println("\nCalling the encoder...\n");
	try {
	    coder.encode(myCard, sink);
	} catch (Exception e) {
	    System.out.println(e);
	    System.exit(1);
	}

	/*****************************************/
	/* Print the encoded data in hexadecimal */
	/*****************************************/
	byte[] encoding = sink.toByteArray();
	System.out.println("\nThe BER encoding is:");
	com.oss.util.HexTool.printHex(encoding);

	/************************************/
	/* Call the decoder to decode myMsg */
	/************************************/
	InputStream source = new ByteArrayInputStream(encoding);
	BBCard decodedData = new BBCard();

	System.out.println("\nCalling the decoder...\n");
	try {
	    decodedData = (BBCard)coder.decode(source, decodedData);
	} catch (Exception e) {
	    System.out.println(e);
	    System.exit(2);
	}

	/***********************************************/
	/* Print the decoded value of field a in       */
	/* decimal                                     */
	/***********************************************/
	System.out.print("\nThe decoded value of field 'a' (decimal) is:");
	System.out.println(
	    decodedData.getA());
	System.out.print(
	"The Java class of the BBCard.a component, created by the decoder is: ");
	System.out.println(decodedData.getA().getClass().getName());

	System.out.println("\nDemonstrate how the user class can " +
	    "be created by a non-ascii value:\n");
	decodedData.getA().setValue(
	    new byte[] {(byte)0x12, (byte)0x34, (byte)0xAB});
	System.out.println(
	    decodedData.getA());

	System.out.println("\nDemonstrate how the user class can " +
	    "throw the UserPrintFailedException for an illegal value:\n");
    decodedData.getA().setValue(null);
	System.out.println(
	    decodedData.getA());

	Bcd.deinitialize();

    } /* end main() */
}